#!/usr/bin/env python3
import sys

import click

from git_monorepo.pull_command import pull
from git_monorepo.push_command import push


@click.group()
def main():
    pass


click_pull = click.command("pull")(pull)
click_push = click.command("push")(push)


@click.command("help")
@click.argument("command")
def help(command) -> None:
    with click.Context(main) as ctx:  # type: ignore
        if "pull" == command:
            click.echo(click_pull.get_help(ctx))
        elif "push" == command:
            click.echo(click_push.get_help(ctx))
        else:
            click.echo(f"Unknown command {command}")
            sys.exit(1)


main.add_command(click_pull)
main.add_command(click_push)
main.add_command(help)


if __name__ == "__main__":
    main()



import os
import subprocess

from termcolor_util import yellow

from git_monorepo.project_config import read_config


def pull() -> None:
    monorepo = read_config()

    for folder_name, repo_location in monorepo.repos.items():
        print(
            yellow(repo_location, bold=True),
            yellow("->"),
            yellow(folder_name, bold=True),
        )

        if not os.path.isdir(folder_name):
            subprocess.check_call(
                [
                    "git",
                    "subtree",
                    "add",
                    "-P",
                    folder_name,
                    repo_location,
                    monorepo.current_branch,
                ],
                cwd=monorepo.project_folder,
            )

            continue

        subprocess.check_call(
            [
                "git",
                "subtree",
                "pull",
                "-P",
                folder_name,
                repo_location,
                monorepo.current_branch,
            ],
            cwd=monorepo.project_folder,
        )



import subprocess

from termcolor_util import yellow, green

from git_monorepo.project_config import (
    read_config,
    GitMonorepoConfig,
    write_synchronized_commits,
)


def push():
    monorepo = read_config()
    something_changed = False

    for folder_name, repo_location in monorepo.repos.items():
        if is_repo_unchanged(monorepo, folder_name):
            print(
                green(repo_location, bold=True),
                green("->"),
                green(folder_name, bold=True),
                green("UNCHANGED", bold=True),
            )
            continue

        something_changed = True

        print(
            yellow(repo_location, bold=True),
            yellow("->"),
            yellow(folder_name, bold=True),
            yellow("PUSH", bold=True),
        )
        subprocess.check_call(
            [
                "git",
                "subtree",
                "push",
                "-P",
                folder_name,
                repo_location,
                monorepo.current_branch,
            ],
            cwd=monorepo.project_folder,
        )

    if not something_changed:
        return

    # we need to update the last commit file with the new value
    monorepo.synchronized_commits = [get_current_commit(monorepo)]
    write_synchronized_commits(monorepo)


def is_repo_unchanged(monorepo: GitMonorepoConfig, folder_name: str) -> bool:
    """
    We check if the sub-repo is changed. This is done via a log that could happen
    against multiple branches if this is a merge.
    :param monorepo:
    :param folder_name:
    :return:
    """
    # if no commits are synchronized, we need to mark this repo as changed
    # first, so the changes are being pushed
    if not monorepo.synchronized_commits:
        return False

    for last_commit in monorepo.synchronized_commits:
        folder_log = (
            subprocess.check_output(
                ["git", "log", f"{last_commit}..HEAD", "--", folder_name],
                cwd=monorepo.project_folder,
            )
            .decode("utf-8")
            .strip()
        )

        if folder_log:
            return False

    return True


def get_current_commit(monorepo: GitMonorepoConfig) -> str:
    return (
        subprocess.check_output(
            ["git", "rev-parse", "HEAD"],
            cwd=monorepo.project_folder,
        )
        .decode("utf-8")
        .strip()
    )



import os
import subprocess
import sys
from typing import Dict, Any, List

import yaml
from termcolor_util import red, yellow


class GitMonorepoConfig:
    def __init__(
        self,
        *,
        repos: Dict[str, str],
        current_branch: str,
        synchronized_commits: List[str],
        project_folder: str,
    ) -> None:
        # maps local folder to remote git repo location
        self.repos = repos
        self.current_branch = current_branch
        self.synchronized_commits = synchronized_commits
        self.project_folder = project_folder


def read_config() -> GitMonorepoConfig:
    monorepo_config_folder = os.path.abspath(os.curdir)
    while monorepo_config_folder and not os.path.isfile(
        os.path.join(monorepo_config_folder, "monorepo.yml")
    ):
        parent_folder = os.path.dirname(monorepo_config_folder)

        if parent_folder == monorepo_config_folder:
            print(
                red("Unable to find"),
                red("monorepo.yml", bold=True),
                red("in any of the parents from"),
                red(os.path.abspath(os.curdir), bold=True),
            )
            sys.exit(1)

        monorepo_config_folder = parent_folder

    project_folder = monorepo_config_folder

    current_branch = (
        subprocess.check_output(
            ["git", "rev-parse", "--abbrev-ref", "HEAD"], cwd=project_folder
        )
        .decode(encoding="utf-8")
        .strip()
    )

    config_file_name = os.path.join(project_folder, "monorepo.yml")
    with open(config_file_name, "rt") as f:
        config_data = yaml.safe_load(f)

    synchronized_commits = _read_synchronized_commits(project_folder)

    repos: Dict[str, str] = dict()
    _merge_repos(path="", repos=repos, data=config_data["mappings"])

    return GitMonorepoConfig(
        repos=repos,
        current_branch=current_branch,
        project_folder=project_folder,
        synchronized_commits=synchronized_commits,
    )


def write_synchronized_commits(monorepo: GitMonorepoConfig):
    sync_file_name = os.path.join(monorepo.project_folder, ".monorepo.sync")

    print(yellow("Updating"), yellow(sync_file_name, bold=True))

    with open(sync_file_name, "wt", encoding="utf-8") as f:
        yaml.safe_dump(monorepo.synchronized_commits, f)

    subprocess.check_call(
        ["git", "add", sync_file_name],
        cwd=monorepo.project_folder,
    )
    subprocess.check_call(
        [
            "git",
            "commit",
            "-m",
            f"git-monorepo: sync commit: {monorepo.synchronized_commits}",
        ],
        cwd=monorepo.project_folder,
    )


def _read_synchronized_commits(project_folder: str) -> List[str]:
    sync_file_name = os.path.join(project_folder, ".monorepo.sync")

    if not os.path.isfile(sync_file_name):
        return []

    with open(sync_file_name, "rt", encoding="utf-8") as f:
        return yaml.safe_load(f)


def _merge_repos(*, repos: Dict[str, str], path: str, data: Dict[str, Any]) -> None:
    for key_name, key_value in data.items():
        relative_path = os.path.join(path, key_name)

        if isinstance(key_value, str):
            repos[relative_path] = key_value
            continue

        _merge_repos(
            repos=repos,
            path=relative_path,
            data=key_value,
        )
